
import click
import yaml

from .generate import generate_menu


def generate_digest(items):
    for i in items:
        if isinstance(i, str):
            yield from generate_menu(i)
        elif isinstance(i, dict):
            yield from generate_menu(**i)
        else:
            raise ValueError("Item should be either string or dict")


@click.command()
@click.argument("config", type=click.File())
def print_digest(config):
    """
    Print digest of daily menus for a list of places configured in a YAML file.
    """
    c = yaml.safe_load(config)
    menu = c.get("menu", [])
    for line in generate_digest(menu):
        print(line)




from urllib.error import URLError
import importlib

import click


def generate_menu(
    extractor,
    override_name=None, override_url=None, **kwargs,
):
    """
    Call given extractor. Generate series of Markdown lines with
    menu of given restaurant. Pass any other keyword arguments to
    the extractor.
    Catch most exceptions, write them nicely.
    """
    try:
        e = importlib.import_module(".extractors." + extractor, __package__)
        name = override_name or e.get_name(**kwargs)
        url = override_url or e.get_url(**kwargs)
        title = "[{name}]({url})".format(name=name, url=url)
        yield title
        yield "-"*len(title)
        yield ""
        for meal, price in e.get_menu(**kwargs):
            yield "*  {meal} {price}". format(meal=meal, price=price)
    except ImportError as err:
        yield "Cannot import {extractor}: {err}".format(
            extractor=extractor, err=err,
        )
    except (AttributeError, ValueError, TimeoutError, URLError) as err:
        yield "Error: {err}".format(err=err)
    yield ""


@click.command()
@click.option("--place-id", "-i", help="Extractor-specific place ID")
@click.option(
    "--option", "-o", help="Extractor-specific key-value option",
    multiple=True, type=(str, str), metavar="<KEY VALUE>…",
)
@click.argument("extractor")
def print_menu(extractor, place_id, option):
    """
    Print daily menu for a place identified by an extractor and
    optional extractor-dependent place_id.
    """
    kwargs = dict(option)
    if place_id:
        kwargs["place_id"] = place_id
    for line in generate_menu(extractor, **kwargs):
        print(line)



import datetime
import locale

from ..utils import parsed_html_doc


def get_name():
    return "Potrefená husa"


def get_url(place_id='dejvice'):
    return "https://www.potrefene-husy.cz/cz/{}-poledni-menu".format(place_id)


def get_menu():
    doc = parsed_html_doc(get_url())
    locale.setlocale(locale.LC_TIME, 'cs_CZ.UTF8')
    dayname = datetime.date.today().strftime("%A")
    rows = doc.xpath(
        '//*[@id="content-in"]//h3[starts-with(text(), "{}")]'
        '/ancestor::tr/following-sibling::tr'.format(dayname),
    )
    for meal in rows:
        cols = meal.findall("td")
        if len(cols) == 3:
            num, name, price = cols
        else:
            return
        if num.text_content().strip() == "":
            return
        if (name is not None) and (price is not None):
            mealname = name.text_content().strip()
            if mealname:
                yield (mealname, price.text_content().strip())



import email.mime.multipart
import email.mime.text
import email.utils
import smtplib
import copy
import datetime

import markdown
import yaml
import click

from .isholiday import isholiday
from .digest import generate_digest
from .version import TAGLINE

htmltemplate = """
<!DOCTYPE html>
<html>
<head>
        <meta charset="utf-8">
        <title>Polední menu</title>
        <style>
            ul {{
                list-style-type: disc;
            }}
        </style>
</head>
<body>
{body}
</body>
</html>
"""


def htmlize(text):
    """Return HTML version of markdown text"""
    htmlized = markdown.markdown(
        text,
        output_format="xhtml5", safe_mode="escape",
    )
    htmlversion = htmltemplate.format(body=htmlized)
    return htmlversion


def send_html_email(
    textpart, htmlpart, subject, recipients,
    server="localhost",
    sender="Foodmaster <foodmaster@localhost>",
):
    """
    Sends multipart alternative e-mail
    """

    msgtpl = email.mime.multipart.MIMEMultipart('alternative')
    msgtpl['Subject'] = subject
    msgtpl['From'] = email.utils.formataddr(email.utils.parseaddr(sender))
    msgtpl['Date'] = email.utils.formatdate(localtime=True)
    msgtpl.attach(email.mime.text.MIMEText(textpart))
    msgtpl.attach(email.mime.text.MIMEText(htmlpart, 'html'))

    with smtplib.SMTP(server) as smtp:
        for r in recipients:
            msg = copy.deepcopy(msgtpl)
            msg['Message-id'] = email.utils.make_msgid('poledni-menu')
            msg['To'] = email.utils.formataddr(email.utils.parseaddr(r))
            smtp.send_message(msg)


def send_email_digest(config):
    dow = (
        "pondělí", "úterý", "středu",
        "čtvrtek", "pátek", "sobotu", "neděli",
    )
    mname = (
        "ledna", "února", "března", "dubna", "května", "června", "července",
        "srpna", "září", "října", "listopadu", "prosince",
    )

    td = datetime.date.today()
    if isholiday(td):
        return
    subject = (
        "\N{Fork and Knife With Plate} Polední nabídka pro"
        " {dow} {day}. {mname}"
    ).format(
        dow=dow[td.weekday()],
        day=td.day,
        mname=mname[td.month-1],
    )
    menu = config.get("menu", [])
    textmenu = "\n".join([
        *generate_digest(menu),
        TAGLINE,
    ])
    htmlmenu = htmlize(textmenu)
    emailargs = config.get("email", {})
    send_html_email(textmenu, htmlmenu, subject, **emailargs)


@click.command()
@click.argument("config", type=click.File())
def email_digest(config):
    """
    E-mail daily menu digest for a list of places.
    """
    send_email_digest(yaml.safe_load(config))



from ..utils import killwhitespace, parsed_html_doc

_BASE_URL = "https://agata.suz.cvut.cz/jidelnicky/index.php?clPodsystem={}"


def get_url(place_id=None):
    return _BASE_URL.format(place_id or "5")


def get_name(place_id=None):
    doc = parsed_html_doc(get_url(place_id))
    h3 = doc.find('//h3')
    if h3 is not None:
        return h3.text_content()
    else:
        return "Neznámá menza"


def get_menu(place_id=None):
    doc = parsed_html_doc(get_url(place_id))
    for meal in doc.findall(
        '//div[@id="jidelnicek"]/div[@class="data"]'
        '/table//tr',
    ):
        tds = meal.findall('td')
        if len(tds) == 0:
            continue
        name = tds[1].text
        price = tds[5].text
        place = tds[6].text_content()
        name, price, place = (killwhitespace(s) for s in (name, price, place))
        if name is "" or name is ".":
            continue
        if place:
            name = "{} ({})".format(name, place)
        yield (name, price)



import datetime
import html

try:
    import requests
except ImportError:
    raise ValueError("This extractor requires requests package to work")


def _get_res_id(place_id=None, res_id=None, api_key=None):
    return res_id or place_id or "18311812"


def _parsed_api_call(
    endpoint="dailymenu", place_id=None,
    res_id=None, api_key=None,
):
    if not hasattr(_parsed_api_call, "cache"):
        _parsed_api_call.cache = dict()
    if api_key is None:
        raise ValueError(
            "This extractor requires an API key. Obtain one and "
            "add it as option called 'api_key'.",
        )
    res_id = _get_res_id(place_id, res_id)
    if (endpoint, res_id) not in _parsed_api_call.cache:
        url = "https://developers.zomato.com/api/v2.1/" + endpoint
        params = {'res_id': res_id}
        headers = {
            'user_key': api_key,
            'Accept': 'application/json',
        }
        _parsed_api_call.cache[(endpoint, res_id)] = requests.get(
            url, headers=headers, params=params,
        ).json()
    return _parsed_api_call.cache[(endpoint, res_id)]


_PLACES = {
    "16505958": ("Pod Loubím", "http://www.podloubim.com/menu/"),
    "18311812": ("Nad Alejí", "https://nadaleji.wixsite.com/nadaleji"),
    "16506914": ("DAP", "http://www.daphotel.cz/"),
}


def get_name(**kwargs):
    doc = _parsed_api_call(endpoint="restaurant", **kwargs)
    return doc.get("name", "Neznámá restaurace")


def get_url(**kwargs):
    res_id = _get_res_id(**kwargs)
    try:
        return _PLACES[res_id][1]
    except KeyError:
        doc = _parsed_api_call(endpoint="restaurant", **kwargs)
        return doc.get("menu_url", "")


def get_menu(**kwargs):
    doc = _parsed_api_call(**kwargs)
    dishes = []
    for menu in doc['daily_menus']:
        sd = menu['daily_menu'].get('start_date')
        ed = menu['daily_menu'].get('end_date')
        if sd:
            sd = datetime.datetime.strptime(sd, "%Y-%m-%d %H:%M:%S")
        if ed:
            ed = datetime.datetime.strptime(ed, "%Y-%m-%d %H:%M:%S")
        now = datetime.datetime.now()
        if sd and sd <= now and (not ed or ed > now):
            dishes.extend(menu['daily_menu'].get('dishes'))

    for meal in dishes:
        name, price = (killthatwhitespace(s)
                       for s in [meal['dish']['name'], meal['dish']['price']])
        yield (name, price)


def killthatwhitespace(string):
    return " ".join(html.unescape(s).strip()
                    for s in string.split()).capitalize()



import urllib.request

import lxml.html


def killwhitespace(string):
    """ Merge consecutive spaces into one space. """
    return " ".join(s.strip() for s in string.split())


def parsed_html_doc(url):
    """ Retrieve HTML document from the URL. Cache results for same URL. """
    if not hasattr(parsed_html_doc, "cache"):
        parsed_html_doc.cache = dict()
    if url not in parsed_html_doc.cache:
        parsed_html_doc.cache[url] = lxml.html.parse(
            urllib.request.urlopen(url),
        )
    return parsed_html_doc.cache[url]



import datetime
import locale

from ..utils import parsed_html_doc


def get_url():
    return "https://nakotlarce.cz/poledni-menu/"


def get_name():
    return "Bernard Pub Na Kotlářce"


def get_menu():
    doc = parsed_html_doc(get_url())
    locale.setlocale(locale.LC_TIME, 'cs_CZ.UTF8')
    dayname = datetime.date.today().strftime("%A")
    daylink = doc.xpath('//h4[text() = "{}"]/parent::a/@href'.format(dayname),)
    if len(daylink) < 1:
        raise ValueError("Jídelní lístek nenalezen")
    daylink = daylink[0][1:]
    meals = doc.findall('//*[@id="{}"]//tr'.format(daylink))
    for meal in meals:
        if len(meal) != 2:
            continue
        name, price = [x.text_content().strip() for x in meal.getchildren()]
        if not price.endswith("Kč"):
            continue
        yield (name, price)



import datetime

from ..utils import parsed_html_doc


def get_name():
    return "Avantgardino"


def get_url():
    return "https://www.avantgardino.cz/"


def get_menu():
    doc = parsed_html_doc(get_url())
    rows = doc.xpath(
        '//div[@class="dennimenu"]'
        '/h3[contains(text(), "{0.day}. {0.month}")]'
        '/../table/tbody/tr'.format(
            datetime.date.today(),
        ),
    )
    for meal in rows:
        cols = meal.findall("td")
        if len(cols) == 3:
            weight, name, price = cols
        else:
            continue
        if (name is not None) and (price is not None):
            mealname = name.text_content().strip().capitalize()
            if mealname:
                yield (mealname, price.text_content().strip())



import string
import urllib.request

import lxml.html


def get_name():
    return "Budvarka"


def get_url():
    return "https://dejvice.pivnice-budvarka.cz/"


def get_menu():
    # They don't have HTTP-EQUIV meta tag with encoding
    parser = lxml.html.HTMLParser(encoding="utf-8")
    doc = lxml.html.parse(urllib.request.urlopen(get_url()), parser)
    menus = doc.xpath('//div[@id="menu-daily"]//dl/dt[@class="menu-item"]')
    for dt in menus:
        name = dt.find('span[@class="name"]').text
        name = name.strip(" " + string.punctuation)
        price = ""
        desc = ""
        dd = dt.getnext()
        while dd is not None and dd.tag == "dd":
            if dd.get("class") == "price":
                price = dd.text
            elif dd.get("class") == "description":
                desc = dd.text.strip()
            dd = dd.getnext()
        yield (" ".join([name, desc]), price)



import urllib.parse
import urllib.request

import lxml.html

_PLACES = {
    "Pod Juliskou": "https://www.podjuliskou.cz/menu/",
}


def get_name(place_id=None):
    return place_id or "Pod Juliskou"


def get_url(place_id=None):
    return _PLACES.get(get_name(place_id), "")


def get_menu(place_id=None):
    """Menu podle Obědáře"""

    url = "http://obedar.fit.cvut.cz/restaurants/{}".format(
        urllib.parse.quote(get_name(place_id)),
    )
    parser = lxml.html.HTMLParser(encoding="utf-8")
    doc = lxml.html.parse(urllib.request.urlopen(url), parser)
    for meal in doc.findall('//table/tr'):
        name = meal.find('td')
        price = meal.find('td[@class="price"]')
        if name is None or price is None:
            continue
        name, price = (el.text_content() for el in (name, price))
        yield (name, price)



import datetime


def getholidays(year):
    """Return a set public holidays in CZ as (day, month) tuples."""

    holidays = {
        (1, 1),   # Novy rok
        (1, 5),   # Svatek prace
        (8, 5),   # Den vitezstvi
        (5, 7),   # Cyril a Metodej
        (6, 7),   # Jan Hus
        (28, 9),  # Den Ceske statnosti
        (28, 10),  # Vznik CS statu
        (17, 11),  # Den boje za svobodu a dem.
        (24, 12),  # Stedry den
        (25, 12),  # 1. svatek Vanocni
        (26, 12),  # 2. svatek Vanocni
    }

    # Easter holiday LUT source:
    # http://www.whyeaster.com/customs/dateofeaster.shtml
    easterlut = [
        (4, 14), (4, 3), (3, 23), (4, 11), (3, 31), (4, 18), (4, 8),
        (3, 28), (4, 16), (4, 5), (3, 25), (4, 13), (4, 2), (3, 22),
        (4, 10), (3, 30), (4, 17), (4, 7), (3, 27),
    ]
    easterday = datetime.date(year, *easterlut[year % 19])
    easterday += datetime.timedelta(6 - easterday.weekday())
    holidays.update(((d.day, d.month) for d in [
        easterday - datetime.timedelta(2),
        easterday + datetime.timedelta(1),
    ]))
    return holidays


def isholiday(date):
    return (date.day, date.month) in getholidays(date.year)



from poledni_menu import version
from setuptools import setup
from pathlib import Path

readme = Path(__file__).with_name("README.rst").read_text()


setup(
    name="poledni-menu",
    version=version.VERSION,
    description="Scrap daily menu and send it via e-mail",
    long_description=readme,
    long_description_content_type="text/x-rst",
    url=version.URL,
    author="Ondřej Caletka",
    author_email="ondrej@caletka.cz",
    license="MIT",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Utilities",
    ],
    packages=["poledni_menu", "poledni_menu.extractors"],
    install_requires=[
        "click",
        "lxml",
        "pyyaml",
        "markdown",
    ],
    setup_requires=["pytest-runner"],
    tests_require=["pytest-vcr"],
    entry_points={
        "console_scripts": [
            "poledni-menu-print = poledni_menu.generate:print_menu",
            "poledni-menu-digest = poledni_menu.digest:print_digest",
            "poledni-menu-email = poledni_menu.email_menu:email_digest",
        ],
    },
)



import datetime
import locale

from ..utils import parsed_html_doc


def get_name():
    return "Kulaťák"


def get_url():
    return "https://www.kulatak.cz/"


def get_menu():
    doc = parsed_html_doc(get_url())
    locale.setlocale(locale.LC_TIME, 'cs_CZ.UTF8')
    dayname = datetime.date.today().strftime("%A")
    for meal in doc.xpath('//div[@id="daily_menu"]/table/tbody/tr/td'
                          '/strong[contains(.,"{}")]/ancestor::tr'
                          '/following-sibling::tr[position()<5]'
                          ''.format(dayname)):
        cells = meal.findall('td')
        if len(cells) != 3:
            continue
        x, name, price = cells
        if (name is not None) and (price is not None):
            yield (
                name.text_content().strip().capitalize(),
                price.text_content().strip(),
            )



VERSION = "0.15"
URL = "https://github.com/oskar456/poledni-menu"
TAGLINE = (
    "Vygenerováno nástrojem [poledni-menu]({URL}) verze {VERSION}. "
    "Pull requesty jsou vítány."
).format(URL=URL, VERSION=VERSION)



import datetime
import locale

from ..utils import parsed_html_doc


def get_name():
    return "Blox"


def get_url():
    return "http://www.blox-restaurant.cz/#!/page_obedy"


def get_menu():
    doc = parsed_html_doc(get_url())
    locale.setlocale(locale.LC_TIME, 'cs_CZ.UTF8')
    dayname = datetime.date.today().strftime("%A")
    rows = doc.xpath(
        '//*[@id="page_obedy"]//*[starts-with(text(), "{}")]'
        '/ancestor::tr/following-sibling::tr'.format(dayname),
    )
    for meal in rows:
        cols = meal.findall("td")
        if len(cols) == 4:
            num, name, alergens, price = cols
        else:
            continue
        if num.text_content().strip() == "":
            return
        if (name is not None) and (price is not None):
            mealname = name.text_content().strip()
            if mealname:
                yield (mealname, price.text_content().strip())
